#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "heap.h"

#define INF 	0x3F3F3F3F

/**************************************************************
Designation 			Sides	Layers
								(total)	Diameter	Capacity
										(cm)	(GB) 	(GiB)
DVD-1[21] 	SS SL	 	1 		1 		08 		1.46 	1.36
DVD-2		SS DL	 	1 		2 		08 		2.66 	2.47
DVD-3	 	DS SL	 	2 		2 		08 		2.92 	2.72
DVD-4	 	DS DL	 	2 		4 		08 		5.32 	4.95
DVD-5	 	SS SL	 	1 		1 		12		4.70 	4.38
DVD-9	 	SS DL	 	1 		2 		12		8.54 	7.95
DVD-10	 	DS SL	 	2 		2 		12		9.40 	8.75
DVD-14[22] 	DS DL/SL 	2 		3 		12		13.24 	12.33
DVD-18	 	DS DL	 	2 		4 		12		17.08 	15.90
***************************************************************/

enum {
	_08mm1s1l,
	_08mm1s2l,
	_08mm2s2l,
	_08mm2s4l,
	_12mm1s1l,
	_12mm1s2l,
	_12mm2s2l,
	_12mm2s3l,
	_12mm2s4l,

};

const char *dvdtypes[] = {
	"8mm1s1l",
	"8mm1s2l",
	"8mm2s2l",
	"8mm2s4l",
	"12mm1s1l",
	"12mm1s2l",
	"12mm2s2l",
	"12mm2s3l",
	"12mm2s4l",

};

const int dvdsizes[] = {
	 1360000,
	 2470000,
	 2720000,
	 4950000,
	 4382000,
	 7950000,
	 8750000,
	12330000,
	15900000,
};

/* Contents of each file */
struct file_el{
	char *name;
	size_t size;
};

/* Contents of each dvd */
struct file_list{
	struct file_el **list;
	size_t n;
};

/* List of dvds */
struct dvd_list{
	struct file_list *list;
	size_t *sum_list;
	size_t n;
};

void usage(){
	fprintf( stderr,
			"options:\n"
			"-d to chose one of the folowing formats and sizes:\n" );
	size_arguments();
	fprintf( stderr,
			"-h to show this help\n"
			"-s to specify a numerical size.\n"
			"-f to specify the folder with the files\n");
}

void size_arguments(){
 fprintf( stderr,
   "                    size   capacity  \n"
   " name  sides layers (mm) nominal real\n"
   " 8mm1s1l  SS SL     08   1.46   1.36 \n"
   " 8mm1s2l  SS DL     08   2.66   2.47 \n"
   " 8mm2s2l  DS SL     08   2.92   2.72 \n"
   " 8mm2s4l  DS DL     08   5.32   4.95 \n"
   "12mm1s1l  SS SL     12   4.70   4.38 * \n"
   "12mm1s2l  SS DL     12   8.54   7.95 \n"
   "12mm2s2l  DS SL     12   9.40   8.75 \n"
   "12mm2s3l  DS DL/SL  12  13.24  12.33 \n"
   "12mm2s4l  DS DL     12  17.08  15.90 \n"
   "\n\n"
     );
}

char cmp( const void*_1, const void*_2 ){
	struct file_el *v1 = (struct file_el*) _1;
	struct file_el *v2 = (struct file_el*) _2;

	return v1->size > v2->size;
}

int main(int argc, char **argv){
	FILE *pfile = NULL;
	struct dvd_list dvds;		/* list dvds to be made */
	struct file_list unfit;		/* unable to place in any dvd */
	char cmd[255] = "du -a --max-depth=1 ";
	struct heap h;
	int n,i,c;
	size_t maxsize = dvdsizes[_12mm1s1l];

	memset( &dvds, 0, sizeof(dvds) );
	memset( &unfit, 0, sizeof(unfit) );

	heap_init( &h, cmp );
	while( (c = getopt( argc, argv,"d:f:hs:") ) != -1 ){
		switch( c ){
			int i;
			int type_err;
			char buf[255];

			case '?': fprintf(stderr, "Unrecognized option: -%c\n", optopt); break;
			case 'd':
				type_err=1;
				for( i=0; i < sizeof( dvdsizes)/sizeof(*dvdsizes); i++ ){
					if( strcmp( dvdtypes[i], optarg ) == 0 ){
						maxsize = dvdsizes[i];
						type_err=0;
						break;
					}
				}

				if( type_err ){
					fprintf(stderr,
					"plese specify one of the folowing formats or -s for numeric size:\n");
					size_arguments();
					return 1;
				}
			break;
			case 'f':
				strcat( cmd, optarg );
				pfile = popen( cmd, "r" );
				if( pfile == NULL ){
					printf("pfile == NULL!\n" );
					break;
				}
			break;
			case 'h': usage(); break;
			case 's': sscanf( optarg, " %lu", &maxsize );	break;
		}
	}

	if( pfile == NULL ){
		printf("please specify a folder with -f\n");
		return 1;
	}
	for( n=0;; n++ ){
		size_t size;
		char str[255] = "";
		struct file_el *pfl = NULL;

		if(fscanf( pfile, " %lu %[^\n]", &size, str ) == 2 ){
			pfl = malloc( sizeof( struct file_el) );
			pfl->name = malloc( strlen(str)*sizeof(char)+1 );
			strcpy( pfl->name, str );
			pfl->size = size;
			heap_push( &h, pfl );
		}
		else
			break;
	}

	dvds.list = realloc( dvds.list, sizeof(dvds.list[0])*(++dvds.n) );
	dvds.sum_list = realloc( dvds.sum_list, sizeof(dvds.sum_list[0])*(dvds.n) );
	memset( &dvds.list[dvds.n -1], 0, sizeof(dvds.list[dvds.n -1]) );
	memset( &dvds.list[dvds.n -1], 0, sizeof(dvds.list[dvds.n -1]) );

	while( !heap_empty( &h ) ){
		struct file_el *el = NULL;
		if( el == NULL )
			el = heap_pop( &h );
		if( el->size > maxsize ){ /* files bigger then a dvd */
			unfit.list = realloc( unfit.list, (++unfit.n)*sizeof(unfit.list[0]) );
			unfit.list[unfit.n-1] = el;
		}
		else{ /* fit in a dvd */
			int i;
			/* check all dvd's if they have space for el */
			for( i=0; i < dvds.n; i++ ){
				if( maxsize > dvds.sum_list[i] + el->size ){
					struct file_list *fl;
					fl = &dvds.list[i];
					/* i'th dvd recieves an element at it's end.
					 * with is list.n from the correspondent dvd
					 */
					fl->list = realloc( fl->list, (++fl->n)*sizeof(fl->list[0]) );
					fl->list[ fl->n-1 ] = el;
					dvds.sum_list[i] += el->size;
					el = NULL;
					break;
				}
			}
			if( el != NULL ){
				/* file don't fit anywhere, create a new dvd */
				dvds.list = realloc( dvds.list, sizeof(*dvds.list)*(++dvds.n) );
				dvds.sum_list = realloc( dvds.sum_list, sizeof(*dvds.sum_list)*(dvds.n) );
				memset( &dvds.list[dvds.n -1], 0, sizeof(dvds.list[dvds.n -1]) );
				memset( &dvds.sum_list[dvds.n -1], 0, sizeof(dvds.sum_list[dvds.n -1]) );
			}
		}
	}
	printf( "#----------------------------------------------------------\n"
			"# number of dvds:%lu\n", dvds.n );
	printf(	"#----------------------------------------------------------\n");
	printf(	"#----------------------------------------------------------\n"
			"#                    files of dvds                         \n"
			"#----------------------------------------------------------\n");

	for( i=0; i < dvds.n; i++ ){
		int j;
		for( j=0; j < dvds.list[i].n; j++ ){
			printf("%lu \"%s\"\n", dvds.list[i].list[j]->size, dvds.list[i].list[j]->name );
		}
		printf(
				"#----------------------------------------------------------\n");
	}
	
	if( unfit.n != 0 ){
		printf(	"#----------------------------------------------------------\n"
				"#              files bigger then one dvd					\n"
				"#----------------------------------------------------------\n");
		for( i=0; i < unfit.n; i++ )
			printf("%lu \"%s\"\n", unfit.list[i]->size, unfit.list[i]->name );
	}
	return 0;
}

